using System;
using System.Data;
using System.Reflection;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace Prova5
{
    class DataSetConverter : Prova2.DataSetConverterBase
    {
        public DataSetConverter(string assemblyFileName)
            : base(assemblyFileName)
        {
        }

        public override IList GetLists()
        {
            ArrayList lista = new ArrayList();
            foreach (DataTable table in DataSet.Tables)
            {
                Type type = GetTypeFor(table);
                if (type != null)
                {
                    lista.Add(PrepareList(table, type));
                }
            }
            return lista;
        }

        protected override Type FindDataSet()
        {
            foreach (Type type in Assembly.GetTypes())
            {
                if (type.IsSubclassOf(typeof(DataSet)))
                    return type;
            }
            return null;
        }

        protected override Type GetTypeFor(DataTable table)
        {
            foreach (Type type in Assembly.GetTypes())
            {
                if (CheckTypeFor(table, type))
                    return type;
            }
            return null;
        }

        protected override IList PrepareList(DataTable table, Type type)
        {
            if (!CheckTypeFor(table, type))
                throw new ArgumentException("!CheckTypeFor(table, type)");
            ArrayList lista = new ArrayList();
            foreach (DataRow dataRow in table.Rows)
            {
                object instance = Activator.CreateInstance(type);
                foreach (DataColumn column in table.Columns)
                {
                    PropertyInfo propertyInfo = type.GetProperty(column.ColumnName);
                    object value = dataRow[column];
                    propertyInfo.SetValue(instance, value, null);
                }
                lista.Add(instance);
            }
            return lista;
        }

        protected override bool CheckTypeFor(DataTable table, Type type)
        {
            if (type.IsAbstract)
                return false;
            if (type.GetConstructor(Type.EmptyTypes) == null)
                return false;
            if (table.Columns.Count > type.GetProperties().Length)
                return false;
            foreach (DataColumn column in table.Columns)
            {
                PropertyInfo propertyInfo = type.GetProperty(column.ColumnName);
                if (propertyInfo == null || propertyInfo.PropertyType != column.DataType ||
                    !propertyInfo.CanRead || !propertyInfo.CanWrite) return false;
            }
            return true;
        }
    }
}
